#include "unionimap.h"
/**
 *	print_usage()
 *	Function used to print out usage information 
 *	when an error is encountered
 */
void print_usage()
{
		fprintf(stderr,"Usage: unionfsimap [-f] [-r ARG:] filename path\n");
		fprintf(stderr," flag -f: create forward map with name filename\n");
		fprintf(stderr," flag -r ARG: create reverse map with name filename and path and add it to forwardmap ARG\n");
		fprintf(stderr," flag -h: prints this message then quits\n");
		fprintf(stderr," please note that -f -r and -a are mutually exclusive\n");
}
/**
 * verify_forwardmap(char *forwardmap)
 * verifies that the forward map is valid.
 *
 */
int verify_forwardmap(char *forwardmap)
{
		int bytesread = 0,err = 0;
		FILE *fwrdmap = NULL;
		struct fmaphdr header;

		fwrdmap = fopen(forwardmap,"r");
		if(!fwrdmap){
			err =  -EINVAL;
			goto out;
		}
		bytesread = fread((void*)&header,sizeof(struct fmaphdr),1,fwrdmap);
		if(bytesread < sizeof(struct fmaphdr)){
			err =  -EINVAL;
			goto out;
		}
		if(header.magic != FORWARDMAP_MAGIC || header.version != FORWARDMAP_VERSION){
			err = -EINVAL;
			goto out;
		}
out:
		if(fwrdmap){
			fclose(fwrdmap);
		}
		return err;
}
/**
* create_forwardmap(char *filename)
* verifies that the forward map is valid.
*
*/
int create_forwardmap(char *filename)
{
		int byteswritten = 0,err = 0;
		FILE *fwrdmap = NULL;
		struct fmaphdr header;
		uuid_t uuid;
		
		fwrdmap = fopen(forwardmap,"r");
		if(!fwrdmap){
			err =  -EINVAL;
			goto out;
		}
		header.version = FORWARDMAP_VERSION;
		header.magic = FORWARDMAP_MAGIC;
		header.usedbranches = 0;
		uuid_generate(uuid);
		memcpy(&header.uuid,&uuid,UUID_LEN);

		byteswritten = fwrite((void *)&header,sizeof(struct fmaphdr),1,fwrdmap);
		if(byteswritten < sizeof(struct fmaphdr)){
				unlink(filename);
				perror("Write failed: ");
				goto out;
		}	
out:
		if(fwrdmap)
				fclose(fwrdmap);
		return err;
}
int create_reversemap(char *filename, char* path, char* forwardmap)
{
		return 0;
}
int dump_map(char *filename, int debug_level)
{
}
int main(int argc, char** argv)
{
		int c = 0; //Temp varliable to hold our character for the switch
		int cflag = 0,aflag = 0,dflag = 0;
		int err = 0;
		char *avalue = NULL;
		if(argc < 2)
		{
				print_usage();
				err = -EINVAL;
				goto out;
		}
		while ((c = getopt (argc, argv, "a:cd")) != -1)
		{
				switch(c)
				{
						case 'c':
								cflag = c;
								break;
						case 'a':
								aflag = 1;
								avalue = (char*)malloc(sizeof(char)*(strlen(optarg)+1));
								strcpy(avalue,optarg);
								break;
						case 'd':
								dflag++;
								break;
						case '?':
								if(isprint (optopt))
										fprintf (stderr, "Unknown option `-%c'.\n", optopt);
								else
										fprintf (stderr,"Unknown option character `\\x%x'.\n",optopt);
								print_usage();
								err = -EINVAL;
								goto out;
								break;
						default:
								print_usage();
								err = -EINVAL;
								goto out;
								break;	
				}
		}
		if(optind == argc){
			print_usage();
			err = -EINVAL;
			goto out;
		}
		if(!(cflag^aflag^dflag)){
			print_usage();
			err = -EINVAL;
			goto out;
		}
		if(cflag){
				err = create_forwardmap(argv[optind]);
				if(err){
					unlink(argv[optind]);
				}
				goto out;
		} else if(aflag){
				if(optind+1 > argc){
					print_usage();
					err = -EINVAL;
					goto out;
				}
						
				err = create_reversemap(argv[optind],argv[optind+1],avalue);
				if(err){
					unlink(argv[optind]);
					goto out;
				}
				
		} else if(dflag){
				err = dump_map(argv[optind],dflag);
				if(err){
					goto out;
				}
		}
out:
		if(avalue){
			free(avalue);
		}
		return err;
}

